searchData={"content_type":"text/markdown","items":[{"doc":"Ollamex is an Elixir wrapper of [ollama](https://ollama.ai/)'s REST API. This is the main module that contains high-level functions that the user will typically interact with.\n\nOllamex is written based on the [ollama REST API documentation](https://github.com/jmorganca/ollama/blob/main/docs/api.md) for the following endpoints:\n\n* [List Local Models](https://github.com/jmorganca/ollama/blob/main/docs/api.md#list-local-models)\n* [Generate a completion](https://github.com/jmorganca/ollama/blob/main/docs/api.md#generate-a-completion)\n* [Generate a chat completion](https://github.com/jmorganca/ollama/blob/main/docs/api.md#generate-a-chat-completion)\n* [Generate embeddings](https://github.com/jmorganca/ollama/blob/main/docs/api.md#generate-embeddings)\n\nTested with ollama version 0.1.20.","ref":"Ollamex.html","title":"Ollamex","type":"module"},{"doc":"#","ref":"Ollamex.html#module-examples","title":"Examples - Ollamex","type":"module"},{"doc":"```elixir\niex> api = Ollamex.API.new()\n%Ollamex.API{\n  uri: \"http://localhost:11434/api\",\n  models: [\n    %Ollamex.LLModel{\n      name: \"llama2:latest\",\n      digest: \"78e26419b4469263f75331927a00a0284ef6544c1975b826b15abdaef17bb962\",\n      modified_at: \"2024-01-09T22:24:14.925918123+02:00\",\n      size: 3826793677,\n      details: %{\n        \"families\" => [\"llama\"],\n        \"family\" => \"llama\",\n        \"format\" => \"gguf\",\n        \"parameter_size\" => \"7B\",\n        \"quantization_level\" => \"Q4_0\"\n      },\n      modelfile: nil,\n      parameters: nil,\n      template: nil\n    },\n    %Ollamex.LLModel{\n      name: \"mistral:latest\",\n      digest: \"61e88e884507ba5e06c49b40e6226884b2a16e872382c2b44a42f2d119d804a5\",\n      modified_at: \"2024-01-08T17:49:54.570542101+02:00\",\n      size: 4109865159,\n      details: %{\n        \"families\" => [\"llama\"],\n        \"family\" => \"llama\",\n        \"format\" => \"gguf\",\n        \"parameter_size\" => \"7B\",\n        \"quantization_level\" => \"Q4_0\"\n      },\n      modelfile: nil,\n      parameters: nil,\n      template: nil\n    }\n  ],\n  timeout: 120000,\n  errors: []\n  }\n  iex> Ollamex.API.list_models(api)\n  [\"llama2:latest\", \"mistral:latest\"]\n```\n\n#","ref":"Ollamex.html#module-api-initialization","title":"API initialization - Ollamex","type":"module"},{"doc":"```elixir\niex> p = %Ollamex.PromptRequest{model: \"mistral:latest\", prompt: \"Explain using a simple paragraph like I'm 5 years old: Why is the sky not black like space?\"}\n%Ollamex.PromptRequest{\n  model: \"mistral:latest\",\n  prompt: \"Explain using a simple paragraph like I'm 5 years old: Why is the sky not black like space?\",\n  raw: false,\n  format: nil,\n  stream: true,\n  options: nil,\n  images: []\n}\n\niex> Ollamex.generate_with_timeout(p, api)\n{:ok,\n  %Ollamex.LLMResponse{\n  context: [733, 16289, 28793, ...],\n  created_at: \"2024-01-10T19:23:12.943599755Z\",\n  done: true,\n  eval_count: 100,\n  eval_duration: 16850322000,\n  model: \"mistral:latest\",\n  prompt_eval_count: 33,\n  prompt_eval_duration: 2865358000,\n  response: \" The sky isn't black like space because it has [...]\n  pretty colors, and nighttime with stars and the moon!\",\n  total_duration: 24862993618,\n  message: nil,\n  errors: nil\n}}\n```\n\n#","ref":"Ollamex.html#module-generate-a-completion-generate-endpoint","title":"Generate a completion (`/generate` endpoint) - Ollamex","type":"module"},{"doc":"```elixir\nmessages =\n  []\n  |> Ollamex.ChatMessage.append(\"user\", \"why is the sky blue?\")\n  |> Ollamex.ChatMessage.append(\"assistant\", \"due to rayleigh scattering!\")\n  |> Ollamex.ChatMessage.append(\"user\", \"how is that different to Mie scattering?\")\n  |> Enum.map(&Map.from_struct(&1))\niex>\n[\n%{content: \"why is the sky blue?\", images: [], role: \"user\"},\n%{content: \"due to rayleigh scattering!\", images: [], role: \"assistant\"},\n%{\n  content: \"how is that different to Mie scattering?\",\n  images: [],\n  role: \"user\"\n}\n]\n\niex> cr = %Ollamex.ChatRequest{messages: messages, model: \"llama2\", stream: true}\n%Ollamex.ChatRequest{\n  model: \"llama2\",\n  messages: [\n    %{content: \"why is the sky blue?\", images: [], role: \"user\"},\n    %{content: \"due to rayleigh scattering!\", images: [], role: \"assistant\"},\n    %{\n      content: \"how is that different to Mie scattering?\",\n      images: [],\n      role: \"user\"\n    }\n  ],\n  format: nil,\n  options: nil,\n  template: nil,\n  stream: true\n}\niex> Ollamex.chat_with_timeout(cr, api)\n{:ok,\n %Ollamex.LLMResponse{\n  context: nil,\n  created_at: \"2024-01-10T19:29:05.771371091Z\",\n  done: true,\n  eval_count: 515,\n  eval_duration: 83246108000,\n  model: \"llama2\",\n  prompt_eval_count: 61,\n  prompt_eval_duration: 7234332000,\n  response: nil,\n  total_duration: 95606709630,\n  message: %{\n    content: \"Mie scattering is [...] while Rayleigh scattering\n    is responsible for the reddening of sunlight at sunrise\n    and sunset.\",\n    role: \"assistant\"\n  },\n  errors: nil\n}}\n```\n\n#","ref":"Ollamex.html#module-generate-a-chat-completion-chat-endpoint","title":"Generate a chat completion (`/chat` endpoint) - Ollamex","type":"module"},{"doc":"```elixir\niex> p = %Ollamex.PromptRequest{model: \"llama2\", prompt: \"Explain the main features and benefits of the Elixir programming language in a single, concise paragraph.\"}\n%Ollamex.PromptRequest{\n  model: \"llama2\",\n  prompt: \"Explain the main features and benefits of the Elixir programming language in a single, concise paragraph.\",\n  raw: false,\n  format: nil,\n  stream: true,\n  options: nil,\n  images: []\n}\niex> Ollamex.embeddings(p, api)\n%Ollamex.LLMResponse{\n  context: nil,\n  created_at: nil,\n  done: nil,\n  eval_count: nil,\n  eval_duration: nil,\n  model: \"llama2\",\n  prompt_eval_count: nil,\n  prompt_eval_duration: nil,\n  response: nil,\n  total_duration: nil,\n  message: nil,\n  embedding: [-1.6268974542617798, -1.4279855489730835, -0.46105068922042847,\n  0.7557640671730042, -0.17748284339904785, ...],\n  errors: nil\n}\n```","ref":"Ollamex.html#module-generate-embeddings-embeddings-endpoint","title":"Generate embeddings (`/embeddings` endpoint) - Ollamex","type":"module"},{"doc":"[Generate a chat completion](https://github.com/jmorganca/ollama/blob/main/docs/api.md#generate-a-chat-completion) using the `/chat` endpoint of the ollama API.\n\nNote that this doesn't guard against situations in which the LLM keeps generating nonsense forever, such as a stream of newlines or tab characters.","ref":"Ollamex.html#chat/2","title":"Ollamex.chat/2","type":"function"},{"doc":"Same functionality as `chat/2`, but will shutdown the task after the provided `timeout` (in milliseconds, default value `120_000`).","ref":"Ollamex.html#chat_with_timeout/3","title":"Ollamex.chat_with_timeout/3","type":"function"},{"doc":"[Generate embeddings](https://github.com/jmorganca/ollama/blob/main/docs/api.md#generate-embeddings) from an LLM using the `/embeddings` endpoint of the ollama API.","ref":"Ollamex.html#embeddings/2","title":"Ollamex.embeddings/2","type":"function"},{"doc":"Same functionality as `embeddings/2`, but will shutdown the task after the provided `timeout` (in milliseconds, default value `120_000`).","ref":"Ollamex.html#embeddings_with_timeout/3","title":"Ollamex.embeddings_with_timeout/3","type":"function"},{"doc":"[Generate a completion](https://github.com/jmorganca/ollama/blob/main/docs/api.md#generate-a-completion) using the `/generate` endpoint of the ollama API.\n\nNote that this doesn't guard against situations in which the LLM keeps generating nonsense forever, such as a stream of newlines or tab characters.","ref":"Ollamex.html#generate/2","title":"Ollamex.generate/2","type":"function"},{"doc":"Same functionality as `generate/2`, but will shutdown the task after the provided `timeout` (in milliseconds, default value `120_000`).","ref":"Ollamex.html#generate_with_timeout/3","title":"Ollamex.generate_with_timeout/3","type":"function"},{"doc":"Defines the struct for the parameters of the ollama REST API and provides helper functions for initializing a struct, updating the list of models, and generating a list of models available by the API.","ref":"Ollamex.API.html","title":"Ollamex.API","type":"module"},{"doc":"Returns a flat list of available LLMs from the ollama REST API `api`.","ref":"Ollamex.API.html#list_models/1","title":"Ollamex.API.list_models/1","type":"function"},{"doc":"Lazy way of creating a new struct with default values (incl. an URL of `http://localhost:11434/api`), instead of `%Ollamex.API{}`, that also fetches the list of available LLMs from the ollama REST API.","ref":"Ollamex.API.html#new/0","title":"Ollamex.API.new/0","type":"function"},{"doc":"Same as `new/0`, but can be provided with a string representing the full URL of an ollama REST API.","ref":"Ollamex.API.html#new/1","title":"Ollamex.API.new/1","type":"function"},{"doc":"Returns the provided `%Ollama.API{}` struct, enriched with the available LLMs from the ollama REST API `api` as a list of `%Ollama.LLModel{}` structs in the `:models` parameter.","ref":"Ollamex.API.html#update_models/1","title":"Ollamex.API.update_models/1","type":"function"},{"doc":"Defines the struct used to store data of a chat message and provides helper functions to create a list of chat messages using the pipeline operator.","ref":"Ollamex.ChatMessage.html","title":"Ollamex.ChatMessage","type":"module"},{"doc":"Append a message (`%Ollamex.ChatMessage{}`) to a list of existing ones.","ref":"Ollamex.ChatMessage.html#append/4","title":"Ollamex.ChatMessage.append/4","type":"function"},{"doc":"Create the first message (`%Ollamex.ChatMessage{}`) in a list.","ref":"Ollamex.ChatMessage.html#create/3","title":"Ollamex.ChatMessage.create/3","type":"function"},{"doc":"Defines the struct of a request to the `/chat` endpoint of the ollama API.","ref":"Ollamex.ChatRequest.html","title":"Ollamex.ChatRequest","type":"module"},{"doc":"Module containing helper functions used by the other modules of Ollamex.","ref":"Ollamex.Helpers.html","title":"Ollamex.Helpers","type":"module"},{"doc":"Consolidates a list of `%Ollamex.LLMResponse{}` structs into a single response, dealing with `stream: false` and `stream: true`, regardless of the origin of the responses (`/chat` or `/generate`).","ref":"Ollamex.Helpers.html#consolidate_responses/1","title":"Ollamex.Helpers.consolidate_responses/1","type":"function"},{"doc":"Creates an async task from the provided function `fun` with arguments `args`.","ref":"Ollamex.Helpers.html#create_task/2","title":"Ollamex.Helpers.create_task/2","type":"function"},{"doc":"Because the body of the responses of the `/generate`, `/chat` and `/embeddings` endpoints of the Ollama REST API return different data structures, this helper function detects the endpoint from which a list of responses originated.","ref":"Ollamex.Helpers.html#detect_endpoint/1","title":"Ollamex.Helpers.detect_endpoint/1","type":"function"},{"doc":"Extracts the chat message fragments from a list of `%Ollamex.LLMResponse{}` from the `/chat` endpoint and concatenates them into a string.","ref":"Ollamex.Helpers.html#extract_messages/1","title":"Ollamex.Helpers.extract_messages/1","type":"function"},{"doc":"Extracts the response fragments from a list of `%Ollamex.LLMResponse{}` from the `/generate` endpoint and concatenates them into a string.","ref":"Ollamex.Helpers.html#extract_responses/1","title":"Ollamex.Helpers.extract_responses/1","type":"function"},{"doc":"","ref":"Ollamex.Helpers.html#handle_response/1","title":"Ollamex.Helpers.handle_response/1","type":"function"},{"doc":"Converts a map to a target struct, so that API responses can be converted to Ollamex structs.","ref":"Ollamex.Helpers.html#map_to_struct/2","title":"Ollamex.Helpers.map_to_struct/2","type":"function"},{"doc":"Yields the result of a task if within the provided `timeout` (in milliseconds, default value `120_000`), or shuts the task down.\n\nUtilized by the `Ollamex.generate_with_timeout/2`, `Ollamex.generate_with_timeout/3`, `Ollamex.chat_with_timeout/2` and `Ollamex.chat_with_timeout/3` functions to deal with cases where the LLM keeps going on and on with repetition.","ref":"Ollamex.Helpers.html#yield_or_timeout_and_shutdown/2","title":"Ollamex.Helpers.yield_or_timeout_and_shutdown/2","type":"function"},{"doc":"The struct describing a response from an LLM on ollama, covering the fields that both the `/generate` and the `/chat` endpoints of the REST API return.","ref":"Ollamex.LLMResponse.html","title":"Ollamex.LLMResponse","type":"module"},{"doc":"The struct describing a Large Language Model on ollama.","ref":"Ollamex.LLModel.html","title":"Ollamex.LLModel","type":"module"},{"doc":"Defines the struct of a request to the `/generate` endpoint of the ollama API.","ref":"Ollamex.PromptRequest.html","title":"Ollamex.PromptRequest","type":"module"},{"doc":"# Ollamex\n\nAn Elixir wrapper of [ollama](https://ollama.ai/)'s REST API with a few niceties built-in, such as dealing with endless LLM repetitions through a timeout.\n\nOllamex is written based on the [ollama REST API documentation](https://github.com/jmorganca/ollama/blob/main/docs/api.md) for the following endpoints:\n\n* [List Local Models](https://github.com/jmorganca/ollama/blob/main/docs/api.md#list-local-models)\n* [Generate a completion](https://github.com/jmorganca/ollama/blob/main/docs/api.md#generate-a-completion)\n* [Generate a chat completion](https://github.com/jmorganca/ollama/blob/main/docs/api.md#generate-a-chat-completion)\n\nThe [primary motivation](https://overbring.com/blog/2024-01-10-ollamex-ollama-api-elixir-released/) for this simple Elixir wrapper was to use a timeout and avoid situations in which the LLM gets stuck generating a stream of `\\n`, `\\t` and whitespace.","ref":"readme.html","title":"README","type":"extras"},{"doc":"The package is [available in Hex](https://hex.pm/packages/ollamex) and can be installed by adding `ollamex` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ollamex, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation","title":"Installation - README","type":"extras"},{"doc":"The docs can be found at  .","ref":"readme.html#documentation","title":"Documentation - README","type":"extras"},{"doc":"The homepage of Ollamex can be found at [overbring.com](https://overbring.com/software/ollamex).","ref":"readme.html#homepage","title":"Homepage - README","type":"extras"}]}